//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//


const BLOGAPI_ERROR = -1;
const BLOGAPI_OK = 0;

/////////////////////////////////////////////////////////////////////////////////

function blogapi_xmlrpc() {
    var listener = arguments[0];   
    var url = arguments[1];   
    var call = arguments[2];   

    var params = new Array();

    for(var i=3;i<arguments.length;++i)
    {
        params[params.length] = arguments[i];
    }
    var rpc = new JSXMLRPC();

    rpc.call(listener, url, call, params);
}

/////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////

/*
 * blogger.newPost
 * Description: Creates a new post, and optionally publishes it. 
 * Parameters: String appkey, String blogid, String username, String password, String content, boolean publish
 * Return value: on success, String postid of new post; on failure, fault
*/

function blogger_newPost(listener, url, appkey, blogid, username, password, content, publish) {
  return blogapi_xmlrpc(listener, url, "blogger.newPost", appkey, blogid, username, password, content, publish);
}

/*
 * blogger.editPost
 * Description: Updates the information about an existing post. 
 * Parameters: String appkey, String postid, String username, String password, String content, boolean publish
 * Return value: on success, boolean true value; on failure, fault
*/

//HELP
function blogger_editPost(listener, url, appkey, postid, username, password, content, publish) {
  return blogapi_xmlrpc(listener, url, "blogger.editPost", appkey, postid, username, password, content, publish);
}

/*
 * blogger.deletePost
 * Description: Deletes a post. 
 * Parameters: String appkey, String postid, String username, String password, boolean publish

 * Return value: on success, boolean true value; on failure, fault
*/

function blogger_deletePost(listener, url, appkey, postid, username, password, publish) {
    //alert(appkey + " " + postid + " " + username + " " + password + " " + publish + " ");
  return blogapi_xmlrpc(listener, url, "blogger.deletePost", appkey, postid, username, password, publish);
}

/*
 * blogger.getRecentPosts
 * Description: Returns a list of the most recent posts in the system. 
 * Parameters: String appkey, String blogid, String username, String password, int numberOfPosts
 * 
 * Return value: on success, array of structs containing ISO.8601 dateCreated, String userid, String postid, String content; on failure, fault
 * Notes: dateCreated is in the timezone of the weblog blogid
 */

function blogger_getRecentPosts(listener, url, appkey, blogid, username, password, numberOfPosts) {
  return blogapi_xmlrpc(listener, url, "blogger.getRecentPosts", appkey, blogid, username, password, numberOfPosts);
}

/*
 * blogger.getUsersBlogs
 * Description: Returns a list of weblogs to which an author has posting privileges. 
 * Parameters: String appkey, String username, String password
 * 
 * Return value: on success, array of structs containing String url, String blogid, String blogName; on failure, fault
*/

function blogger_getUsersBlogs(listener, url, appkey, username, password) {
  return blogapi_xmlrpc(listener, url, "blogger.getUsersBlogs", appkey, username, password);
}
/*
 * blogger.getUserInfo
 * Description: Returns information about an author in the system. 
 * Parameters: String appkey, String username, String password
 * 
 * Return value: on success, struct containing String userid, String firstname, String lastname, String nickname, String email, String url; on failure, fault
 * 
 * Notes: firstname is the Movable Type username up to the first space character, and lastname is the username after the first space character.
*/

function blogger_getUserInfo(listener, url, appkey, username, password) {
  return blogapi_xmlrpc(listener, url, "blogger.getUserInfo", appkey, username, password);
}

 /* 
 * metaWeblog.newPost
 * Description: Creates a new post, and optionally publishes it. 
 * Parameters: String blogid, String username, String password, struct content, boolean publish
 * 
 * Return value: on success, String postid of new post; on failure, fault
 * 
 * Notes: the struct content can contain the following standard keys: 
   title, for the title of the entry; 
   description, for the body of the entry; and 
   dateCreated, to set the created-on date of the entry. 
   
   In addition, Movable Type's implementation allows you to pass in values for five other keys: 
   int mt_allow_comments, the value for the allow_comments field; 
   int mt_allow_pings, the value for the allow_pings field; 
   String mt_convert_breaks, the value for the convert_breaks field; 
   String mt_text_more, the value for the additional entry text; 
   String mt_excerpt, the value for the excerpt field; 
   String mt_keywords, the value for the keywords field; and 
   array mt_tb_ping_urls, the list of TrackBack ping URLs for this entry. If specified, dateCreated should be in ISO.8601 format.
 */

function metaWeblog_newPost(listener, url, blogid, username, password, content_t, publish) {
  return blogapi_xmlrpc(listener, url, "metaWeblog.newPost", blogid, username, password, content_t, publish);
}

/*
 * metaWeblog.editPost
 * Description: Updates information about an existing post. 
 * Parameters: String postid, String username, String password, struct content, boolean publish
 * 
 * Return value: on success, boolean true value; on failure, fault
 * 
 * Notes: the struct content can contain the following standard keys: title, for the title of the entry; description, for the body of the entry; and dateCreated, to set the created-on date of the entry. In addition, Movable Type's implementation allows you to pass in values for five other keys: int mt_allow_comments, the value for the allow_comments field; int mt_allow_pings, the value for the allow_pings field; String mt_convert_breaks, the value for the convert_breaks field; String mt_text_more, the value for the additional entry text; String mt_excerpt, the value for the excerpt field; String mt_keywords, the value for the keywords field; and array mt_tb_ping_urls, the list of TrackBack ping URLs for this entry. If specified, dateCreated should be in ISO.8601 format.
 */
function metaWeblog_editPost(listener, url, postid, username, password, content_t, publish) {
  //return XMLRPC.call(myListener, url, "metaWeblog.editPost", postid, username, password, content_t, publish);
  return blogapi_xmlrpc(listener, url, "metaWeblog.editPost", postid, username, password, content_t, publish);
}

/*
 * 
 * metaWeblog.getPost
 * Description: Returns information about a specific post. 
 * Parameters: String postid, String username, String password
 * 
 * Return value: on success, struct containing String userid, ISO.8601 dateCreated, String postid, String description, String title, String link, String permaLink, String mt_excerpt, String mt_text_more, int mt_allow_comments, int mt_allow_pings, String mt_convert_breaks, String mt_keywords; on failure, fault
 * 
 * Notes: link and permaLink are both the URL pointing to the archived post. The fields prefixed with mt_ are Movable Type extensions to the metaWeblog.getPost API.
 */
function metaWeblog_getPost(listener, url, postid, username, password) {
  return blogapi_xmlrpc(listener, url, "metaWeblog.getPost", postid, username, password);
}

/*
 * metaWeblog.getRecentPosts
 * Description: Returns a list of the most recent posts in the system. 
 * Parameters: String blogid, String username, String password, int numberOfPosts
 * 
 * Return value: on success, array of structs containing ISO.8601 dateCreated, String userid, String postid, String description, String title, String link, String permaLink, String mt_excerpt, String mt_text_more, int mt_allow_comments, int mt_allow_pings, String mt_convert_breaks, String mt_keywords; on failure, fault
 * 
 * Notes: dateCreated is in the timezone of the weblog blogid; link and permaLink are the URL pointing to the archived post
 */
function metaWeblog_getRecentPosts(listener, url, blogid, username, password, numberOfPosts) {
  return blogapi_xmlrpc(listener, url, "metaWeblog.getRecentPosts", blogid, username, password, numberOfPosts);
}

/*
 * metaWeblog.newMediaObject
 * Description: Uploads a file to your webserver. 
 * Parameters: String blogid, String username, String password, struct file
 * 
 * Return value: URL to the uploaded file.
 * 
 * Notes: the struct file should contain two keys: base64 bits (the base64-encoded contents of the file) and String name (the name of the file). The type key (media type of the file) is currently ignored.
 */

/*
function metaWeblog_newMediaObject(listener, url, blogid, username, password, file_t)
{
  return blogapi_xmlrpc(listener, url, "metaWeblog.newMediaObject", blogid, username, password, file_t);
}
*/

/*
 * mt.getRecentPostTitles
 * Description: Returns a bandwidth-friendly list of the most recent posts in the system. 
 * Parameters: String blogid, String username, String password, int numberOfPosts
 * 
 * Return value: on success, array of structs containing ISO.8601 dateCreated, String userid, String postid, String title; on failure, fault
 * 
 * Notes: dateCreated is in the timezone of the weblog blogid
*/

function mt_getRecentPostTitles(listener, url, blogid, username, password, numberOfPosts) {
  return blogapi_xmlrpc(listener, url, "mt.getRecentPostTitles", blogid, username, password, numberOfPosts);
}

/*
 * mt.getCategoryList
 * Description: Returns a list of all categories defined in the weblog. 
 * Parameters: String blogid, String username, String password
 * 
 * Return value: on success, an array of structs containing String categoryId and String categoryName; on failure, fault.
 */
function mt_getCategoryList(listener, url, blogid, username, password) {
  return blogapi_xmlrpc(listener, url, "mt.getCategoryList", blogid, username, password);
}

/*
 * mt.getPostCategories
 * Description: Returns a list of all categories to which the post is assigned. 
 * Parameters: String postid, String username, String password

 * Return value: on success, an array of structs containing String categoryName, String categoryId, and boolean isPrimary; on failure, fault.
 * 
 * Notes: isPrimary denotes whether a category is the post's primary category.
 */

function mt_getPostCategories(listener, url, postid, username, password) {
  return blogapi_xmlrpc(listener, url, "mt.getPostCategories", postid, username, password);
}

/*
 * mt.setPostCategories
 * Description: Sets the categories for a post. 
 * Parameters: String postid, String username, String password, array categories
 * 
 * Return value: on success, boolean true value; on failure, fault
 * 
 * Notes: the array categories is an array of structs containing 
   String categoryId and boolean isPrimary. 
   Using isPrimary to set the primary category is optional
   --in the absence of this flag, 
   the first struct in the array will be assigned the primary category 
   for the post.
 */
function mt_setPostCategories(listener, url, postid, username, password, categories) {
  return blogapi_xmlrpc(listener, url, "mt.setPostCategories", postid, username, password, categories);
}

/*
 * 
 * 
 * metaWeblog.newMediaObject
 * Description: Uploads a file to your webserver. 
 * Parameters: String blogid, String username, String password, struct file
 * 
 * Return value: URL to the uploaded file.
 * 
 * Notes: the struct file should contain two keys: base64 bits (the base64-encoded contents of the file) and String name (the name of the file). The type key (media type of the file) is currently ignored.
 * 
 * 

 * mt.getPostCategories
 * Description: Returns a list of all categories to which the post is assigned. 
 * Parameters: String postid, String username, String password

 * Return value: on success, an array of structs containing String categoryName, String categoryId, and boolean isPrimary; on failure, fault.
 * 
 * Notes: isPrimary denotes whether a category is the post's primary category.
 * 
 * 
 * mt.setPostCategories
 * Description: Sets the categories for a post. 
 * Parameters: String postid, String username, String password, array categories
 * 
 * Return value: on success, boolean true value; on failure, fault
 * 
 * Notes: the array categories is an array of structs containing String categoryId and boolean isPrimary. Using isPrimary to set the primary category is optional--in the absence of this flag, the first struct in the array will be assigned the primary category for the post.
 * 
 * 
 * mt.supportedMethods
 * Description: Retrieve information about the XML-RPC methods supported by the server. 
 * Parameters: none
 * 
 * Return value: an array of method names supported by the server.
 * 
 * 
 * mt.supportedTextFilters
 * Description: Retrieve information about the text formatting plugins supported by the server. 
 * Parameters: none
 * 
 * Return value: an array of structs containing String key and String label. key is the unique string identifying a text formatting plugin, and label is the readable description to be displayed to a user. key is the value that should be passed in the mt_convert_breaks parameter to newPost and editPost.
 * 
 * 
 * mt.getTrackbackPings
 * Description: Retrieve the list of TrackBack pings posted to a particular entry. This could be used to programmatically retrieve the list of pings for a particular entry, then iterate through each of those pings doing the same, until one has built up a graph of the web of entries referencing one another on a particular topic. 
 * Parameters: String postid
 * 
 * Return value: an array of structs containing String pingTitle (the title of the entry sent in the ping), String pingURL (the URL of the entry), and String pingIP (the IP address of the host that sent the ping).
 * 
 * 
 * mt.publishPost
 * Description: Publish (rebuild) all of the static files related to an entry from your weblog. Equivalent to saving an entry in the system (but without the ping). 
 * Parameters: String postid, String username, String password
 * 
 * Return value: on success, boolean true value; on failure, fault
 */




/*
var stuffedDog;
function r2_blog_pre_1()
{
  //stuffedDog = XMLRPC.getService("http://www.stuffeddog.com/speller/speller-rpc.cgi");
  stuffedDog = XMLRPC.getService("http://intranet.roundtwo.com/secure/blog/xmlrpc.php");
  stuffedDog.add("speller.newPost", "check", "", "jes", "anthony", "changeme", "TEST", true);
  XMLRPC.onerror = function(e)
  {
	  alert("error " + e);

    var i=0;

    
	  return true;
  }
  //alert("Anthony");
  window.setTimeout("doCheck()",1);
}

var done = false;
function doCheck(){
	//inp = document.getElementById("sentence");
	result = stuffedDog.check("helloe");
//  alert("here" + result);
	check(result);
  //check("hello");
}

function check(data){
  try
  {
	if(data || data == ""){
		res = "";
		
		for(j=0;j<data.length;j++){	
			res += "<br>Found Error in word: " + data[j].word + "<br>Suggestions:<br>";
			
			arr = data[j].suggestions;
			for(i=0;i<arr.length;i++){
				res += arr[i] + "<br>";
			}
		}
		//resultWindow.innerHTML = ((res == "") ? "No Spelling Errors Found" : res);
		alert(res);
	}
  }
  catch(e)
  {
    alert(e);
  }
}
*/


