function ObjectWrapper(obj) {
    this.wrappedJSObject = obj;
}

function feedUtility(method, args) {
    args.method = method;
    args.window = window;
    var wrappedObject = new ObjectWrapper(args);
    chromeMessenger.send(wrappedObject);
}

Document.prototype.getElementsByClassName = function(className) {
    var classFilter = function (element) {
        return element.className.split(/\s+/).indexOf(className) != -1;
    }
    return Array.filter(this.getElementsByTagName("*"), classFilter);
}

Element.show = function(element) {
    if (typeof element == 'string')
        element = document.getElementById(element);
    element.style.display = '';
}

Element.hide = function(element) {
    if (typeof element == 'string')
        element = document.getElementById(element);
    element.style.display = 'none';
}

function toggleElements(className, show) {
    var elements = document.getElementsByClassName(className);
    for (var i = 0; i < elements.length; i++) {
        var element = elements[i];
        if (show)
            Element.show(element);
        else
            Element.hide(element);
    }
}

function excerptsOnly() {
    toggleElements('postbody', false);
    toggleElements('postexcerpt', true);
}

function headlinesOnly() {
    toggleElements('postexcerpt', false);
    toggleElements('postbody', false);
}

function fullView() {
    toggleElements('postexcerpt', false);
    toggleElements('postbody', true);
}

function filterSource(source) {
    var posts = document.getElementsByClassName('post');
    for (var i = 0; i < posts.length; i++) {
        var post = posts[i];
        if(!source) {
            Element.show(post);
        } else {
            if (post.getAttribute("name") == source) {
                Element.show(post);
            } else {
                Element.hide(post);
            }
        }
    }
}

function loadView(url) {
    document.location = url;
}

function flagPost(url, id, sid) {
    var postSave = document.getElementById(sid);
    var state = postSave.getAttribute("class");

    var args = { url: url, id: id };

    if (state == "unsaved") {
        postSave.setAttribute("class", "saved");
        feedUtility('flagPost', args);
    } else {
        postSave.setAttribute("class", "unsaved");
        feedUtility('unflagPost', args);
    }

    return false;
}

function blogThis(url, title, content) {
    var args = {
        url: url,
        title: decodeURIComponent(title),
        content: decodeURIComponent(content),
//@line 102 "/home/build/tinderbuild/src/flock/mozilla/browser/components/flock/feed/content/scripts/view.js"
        alwaysRaised: false
//@line 104 "/home/build/tinderbuild/src/flock/mozilla/browser/components/flock/feed/content/scripts/view.js"
    }

    feedUtility('blogThis', args);
}

function subscribeFeed(url) {
    feedUtility('subscribeFeed', {});

    var subscription = document.getElementById("subscription");
    if (subscription) {
    	subscription.setAttribute("class", "subscribed");
    }
    
    return false;
}

function markRead(url, id) {
    var post = document.getElementById(id);
   	var classes = post.getAttribute("class").split(" ");
   	for (var c = 0; c < classes.length; c++) {
   		if (classes[c] == "read" || classes[c] == "unread") {
   			classes[c] = "read";
	   		break;
    	}
    }
    post.setAttribute("class", classes.join(" "));
    feedUtility('markRead', { url: url, id: id });
    return false;
}

function markUnread(url, id) {
    var post = document.getElementById(id);
   	var classes = post.getAttribute("class").split(" ");
   	for (var c = 0; c < classes.length; c++) {
   		if (classes[c] == "read" || classes[c] == "unread") {
   			classes[c] = "unread";
	   		break;
    	}
    }
    post.setAttribute("class", classes.join(" "));
    feedUtility('markUnread', { url: url, id: id });
    return false;
}

function markAllAsRead(base_url) {
    base_url = base_url.replace(/^feeds:/, '');
    var urls = base_url.split('&');
    for (var i = 0; i < urls.length; i++) {
        var url = urls[i];
        if (url)
            feedUtility('markAllRead', { urn: decodeURIComponent(url) });
    }

    var checkboxes = document.getElementsByClassName('checkbox');
    for (var i = 0; i < checkboxes.length; i++) {
        var checkbox = checkboxes[i];
        if (checkbox.id.indexOf('read-') == 0) {
            checkbox.checked = true;
            var id = checkbox.id.substr('read-'.length);
            var post = document.getElementById(id);
            var classes = post.getAttribute("class").split(" ");
		   	for (var c = 0; c < classes.length; c++) {
		   		if (classes[c] == "read" || classes[c] == "unread") {
		   			classes[c] = "read";
			   		break;
		    	}
		    }
            post.setAttribute("class", classes.join(" "));
        }
    }
}

function selectColumns(columns) {
	if (columns == 1) {
		document.getElementById("single").setAttribute("class", "selected");
        document.getElementById("double").setAttribute("class", "");
	} else {
		document.getElementById("single").setAttribute("class", "");
        document.getElementById("double").setAttribute("class", "selected");
	}
	toggleColumns(columns);
	feedUtility('setDefaultColumns', { columns: columns });
}

function toggleColumns(columns) {
	if (columns == 1) {
		columns = "single";
	} else {
		columns = "double";
	}
	var content = document.getElementById("content");
	content.setAttribute("class", columns);
}

function selectStyle(style) {
    switch (style) {
        case "excerpts":
        	document.getElementById("full").setAttribute("class", "");
        	document.getElementById("excerpts").setAttribute("class", "selected");
        	document.getElementById("headlines").setAttribute("class", "");
            break;
        case "headlines":
        	document.getElementById("full").setAttribute("class", "");
        	document.getElementById("excerpts").setAttribute("class", "");
        	document.getElementById("headlines").setAttribute("class", "selected");
            break;
        case "full":
        	document.getElementById("full").setAttribute("class", "selected");
        	document.getElementById("excerpts").setAttribute("class", "");
        	document.getElementById("headlines").setAttribute("class", "");
            break;
    }
	togglePostStyle(style);
	feedUtility('setDefaultStyle', { style: style });
}

function togglePostStyle(style) {
    var elements = document.getElementsByClassName("post");
    for (var i = 0; i < elements.length; i++) {
		var element = elements[i];
    	var classes = element.getAttribute("class").split(" ");
    	for (var c = 0; c < classes.length; c++) {
    		if (classes[c] == "full" || classes[c] == "excerpts" || classes[c] == "headlines") {
    			classes[c] = style;
 	    		break;
    		}
    	}
    	element.setAttribute("class", classes.join(" "));
    }
}

function setShowUnread(mode) {
	if (mode == "") {
		mode = false;
	} else {
		mode = true;
	}
	feedUtility('setShowUnread', { show: mode });
}

function setShowRead(mode) {
	if (mode == "") {
		mode = false;
	} else {
		mode = true;
	}
	feedUtility('setShowRead', { show: mode });
}

function setShowFlagged(mode) {
	if (mode == "") {
		mode = false;
	} else {
		mode = true;
	}
	feedUtility('setShowFlagged', { show: mode });
}

var popupManager = {

	XHTMLNS: "http://www.w3.org/1999/xhtml",

	currentPopup: null,
	delay: null,
	interval: 0.6,
	
	findPosition: function(oLink) {
		if (oLink.offsetParent) {
			for (var posX = 0, posY = 0; oLink.offsetParent; oLink = oLink.offsetParent) {
				posX += oLink.offsetLeft;
				posY += oLink.offsetTop;
			}
			return [posX, posY];
		} else {
			return [oLink.x, oLink.y];
		}
	},
	
	getParent: function(el, pTagName) {
		if (el == null) {
			return null;
		}
		// gecko bug, supposed to be uppercase
		else if (el.nodeType == 1 && el.tagName.toLowerCase() == pTagName.toLowerCase()) {
			return el;
		} else {
			return this.getParent(el.parentNode, pTagName);
		}
	},
	
	showPopup: function(e, content) {
	    if (window.event && window.event.srcElement) {
	        lnk = window.event.srcElement
		}
		else if (e && e.target) {
	        lnk = e.target
		}
	    if (!lnk) return;
	
		// lnk is a textnode or an elementnode that's not ins/del
	    if (lnk.nodeType == 3 || (lnk.nodeType == 1 && lnk.tagName.toLowerCase() != 'ins' && lnk.tagName.toLowerCase() != 'del')) {
			// ascend parents until we hit a link
			lnk = this.getParent(lnk, 'a');
		}
		
		this.delay = setTimeout(popupManager.renderPopup, popupManager.interval * 1000, lnk, content);
	},
	
	renderPopup: function(link, content) {
	    if (popupManager.currentPopup) popupManager.hidePopup(popupManager.currentPopup);
	    if (!document.getElementsByTagName) return;
	    
	    content = decodeURIComponent(content);
	
	    var d = document.createElementNS(popupManager.XHTMLNS, 'div');
	    d.className = 'popup';
	
	    var r = d.ownerDocument.createRange();
	    r.selectNodeContents(d);
	    r.deleteContents();
	    var df = r.createContextualFragment(content);
	    d.appendChild(df);
	
	    d.style.width = "35%";
	
	    mpos = popupManager.findPosition(link);
	    mx = mpos[0];
	    my = mpos[1];
	    
	    d.style.left = (mx+10) + 'px';
	    d.style.top = (my+20) + 'px';
	
	    document.getElementsByTagName('body')[0].appendChild(d);
	
	    popupManager.currentPopup = d;
	},
	
	hidePopup: function(e) {
		if (popupManager.delay) clearTimeout(popupManager.delay);
		if (!document.getElementsByTagName) return;
		if (popupManager.currentPopup) {
			document.getElementsByTagName('body')[0].removeChild(popupManager.currentPopup);
			popupManager.currentPopup = null;
		}
	}

}
