flock.require('flock.photo.photo');

flock.photo.api = {};
flock.photo.api.calls = {};

flock.photo.api.supportsCall = function(aCall) {
    return typeof(flock.photo.api.calls[aCall]) != "undefined";
}
const FLOCK_PREF_PHOTO_API_CLASSNAME = "flock.photo.api.classname";
const FLOCK_PREF_PHOTO_API_UINAME = "flock.photo.api.uiname";
const FLOCK_PREF_PHOTO_API_JSFILE = "flock.photo.api.jsfile";
const FLOCK_PREF_PHOTO_API_CID = "flock.photo.api.cid";
const FLOCK_PREF_PHOTO_RESIZE = "flock.photo.resize";
const FLOCK_PREF_PHOTO_RESIZE_DIMENSIONS = "flock.photo.resize.dimensions";
const XMLHTTPREQUEST_CONTRACTID     = '@mozilla.org/xmlextras/xmlhttprequest;1';
const DOMPARSER_CONTRACTID     = '@mozilla.org/xmlextras/domparser;1';
const DICTIONARY_CONTRACTID     = '@mozilla.org/dictionary;1';

const nsIXMLHttpRequest             = Components.interfaces.nsIXMLHttpRequest;
const nsIDOMParser             = Components.interfaces.nsIDOMParser;
const nsIDictionary             = Components.interfaces.nsIDictionary;

const FLOCK_IO_CHUNKSIZE = 65536 * 4;
const FLOCK_THUMB_SIZE = 75;
const FLOCK_PREVIEW_SIZE = 300;

/*
 * UI related calls are registered at the xul layer for now
 */
/*
var PhotoAPICalls = {
    getCall:  function(aMethod) {
        var call = this.table.getCall(aMethod);
        if(!call) {
            throw new Exception("Unsupported Photo API Function");
        }
        return call;
    },
    supportsCall: function(aMethod) {
        return  this.table.supportsCall(aMethod);
    },
    register: function(aTable) {
        this.table = aTable;
    },
    clear: function () {
        this.table = null;
    },
}
*/

/*
 * Prototype base Classes for Photo classes
 */
function PhotoAPI() { }

PhotoAPI.prototype = {
    registerUICalls: function() {},
    isLoggedIn: function() {},
    upload: function(aListener, aPhoto, aParams, aUser) { },
    login: function(aListener) { },
}

function PhotoAPIListener() { }

/*
 * flockIPhotoAPIListener
 */
PhotoAPIListener.prototype = {
    onResult: function(aXML) {},
    onError: function(aError) {},
}

function PhotoAPIUser() { }

PhotoAPIUser.prototype = {
    username: "",
    id: "",
}

function Photo() { }

Photo.prototype = {
    webPageUrl: "",
    thumbnail: "",
    midSizePhoto: "",
    title: "",
}

function flock_photoSetAPI(aUIName, aClassName, aJSFile, aCID) {
    flock_setCharPref(FLOCK_PREF_PHOTO_API_UINAME, aUIName);
    flock_setCharPref(FLOCK_PREF_PHOTO_API_CLASSNAME, aClassName);
    flock_setCharPref(FLOCK_PREF_PHOTO_API_JSFILE, aJSFile);
    flock_setCharPref(FLOCK_PREF_PHOTO_API_CID, aCID);
}

function flock_photoCheckAPIInit() {
    var pref = flock_getCharPref(FLOCK_PREF_PHOTO_API_CLASSNAME);
    if(!pref) {
        flock_photoSetAPI("Flickr", "FlickrCalls", "chrome://browser/content/flock/photo/flickrAPI.js", "@flock.com/?photo-api-flickr;1");
    }
}

function flock_photoAPIRegisterUICalls() {
    flock_photoCheckAPIInit();
    var jsFile = flock_getCharPref(FLOCK_PREF_PHOTO_API_JSFILE);
    flock_loadDynamicScript(jsFile);
    var className = flock_getCharPref(FLOCK_PREF_PHOTO_API_CLASSNAME);
    eval(className + ".register();");
}

///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
// Gratuitous separation from utility functions
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////

function params2Dictionary(aParams) {
    var dict = Components.classes[DICTIONARY_CONTRACTID].createInstance(nsIDictionary);
    for(var p in aParams) {
        var val = Components.classes['@mozilla.org/supports-string;1'].createInstance(Components.interfaces.nsISupportsString);
        val.data = aParams[p];
        dict.setValue(p, val);
    }
    return dict;
}


function flock_photoGetService() {
    const CID = '@mozilla.org/rdf/datasource;1?name=flock-photo';
    const IID = Components.interfaces.flockIPhotoService;
    var svc = Components.classes[CID].getService(IID);
    return svc;
}


function flock_getTmpFile(aFName) {
    var sinkFile = Components.classes["@mozilla.org/file/directory_service;1"]
        .getService(Components.interfaces.nsIProperties)
        .get("ProfD", Components.interfaces.nsIFile);
    sinkFile.append("flock_imagecache");
    if(!sinkFile.exists()) {
        sinkFile.createUnique(1,0700);
    }
    if(aFName) sinkFile.append(aFName);
    return sinkFile;
}

/*
function flock_getSpecFromFile(aURL) {
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file")
        .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var sourceSpec = fileHandler.getFileFromURLSpec(aURL);
    return sourceSpec;
}
*/


function flock_getURLFromFile(aFile) {
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file")
        .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    return fileHandler.getURLSpecFromFile(aFile);
}

function flock_getFileFromPath(aPath) {
    var file = Components.classes["@mozilla.org/file/local;1"]
            .createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(aPath);
    return file;
}

function flock_getFileFromURL(aURL) {
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file")
        .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var sourceSpec = fileHandler.getFileFromURLSpec(aURL);
    var sourceFile = Components.classes["@mozilla.org/file/local;1"]
        .createInstance(Components.interfaces.nsILocalFile);
    sourceFile.initWithFile(sourceSpec);
    return sourceFile;
}

function flock_getBufferredInputStream(aFile) {
    var sourceFileStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
        .createInstance(Components.interfaces.nsIFileInputStream);
    sourceFileStream.init(aFile, 1, 0, false);

    var sourceBufferStream = Components.classes["@mozilla.org/network/buffered-input-stream;1"]
        .createInstance(Components.interfaces.nsIBufferedInputStream);

    sourceBufferStream.init(sourceFileStream, FLOCK_IO_CHUNKSIZE); // write, create, truncate
    return sourceBufferStream;
}

function flock_removeFileBySpec(aSpec, aRecursive) {
    var file = flock_getFileFromURL(aSpec);
    file.remove(aRecursive);
}

function flock_removeFileByPath(aPath, aRecursive) {
    var file = flock_getFileFromPath(aPath);
    file.remove(aRecursive);
}

function flock_getBufferredOutputStream(aFile) {
    var sinkFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                            .createInstance(Components.interfaces.nsIFileOutputStream);
    sinkFileStream.init(aFile, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate


    var sinkBufferStream = Components.classes["@mozilla.org/network/buffered-output-stream;1"]
        .createInstance(Components.interfaces.nsIBufferedOutputStream);
    sinkBufferStream.init(sinkFileStream, FLOCK_IO_CHUNKSIZE); // write, create, truncate
    return sinkBufferStream;
}
function flock_getImageUtils() {
    var CID = '@flock.com/imagescaler;1';
    var IID = Components.interfaces.flockIImageScaler;
    return Components.classes[CID].createInstance(IID);
}


function flock_loadDynamicScript(aURL) {
    var loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                .getService(Components.interfaces.mozIJSSubScriptLoader)
    loader.loadSubScript(aURL);
}

PhotoUploader.prototype = {
    endpoint: null,
    photoParam: "photo",
    setEndpoint: function(aEndpoint) { this.endpoint = aEndpoint; },
    setPhotoParam: function(aPhotoParam) { this.photoParam = aPhotoParam; },
    upload: function(aListener, aPhoto, aParams, aUpload) {
        try {
            var now = new Date();
            var tmpFileName = aPhoto + now;
            tmpFileName = "flock_" + hex_md5(tmpFileName) + ".tmp";

            var ios = Components.classes["@mozilla.org/network/io-service;1"]
                .getService(Components.interfaces.nsIIOService);
            var fileHandler = ios.getProtocolHandler("file")
                .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
            var fileFromSpec = fileHandler.getFileFromURLSpec(aPhoto);

            var boundaryString = 'AaB03x';
            var contentType = "multipart/form-data; boundary=" + boundaryString;
            var boundary = '--' + boundaryString;


            var photoFile = Components.classes["@mozilla.org/file/local;1"]
                .createInstance(Components.interfaces.nsILocalFile);
            photoFile.initWithFile(fileFromSpec);

            var photoStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                .createInstance(Components.interfaces.nsIFileInputStream);
            photoStream.init(photoFile, 1, 0644, 0);

            var tmpFile = flock_getTmpFile(tmpFileName);

            var tmpFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                                    .createInstance(Components.interfaces.nsIFileOutputStream);
            tmpFileStream.init(tmpFile, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate

            var tmpBufferStream = Components.classes["@mozilla.org/network/buffered-output-stream;1"]
                .createInstance(Components.interfaces.nsIBufferedOutputStream);
                
            var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
            converter.charset = "UTF-8";

            tmpBufferStream.init(tmpFileStream, 65536 * 4); 

            tmpBufferStream.write(boundary, boundary.length);
            tmpBufferStream.write("\r\n", 2);

            var fileName = fileFromSpec.leafName;
            if(aUpload) {
                fileName = aUpload.fileName;
            }
            var next = "";
            next = 'Content-Disposition: form-data; name="' + this.photoParam + '"; filename="' + fileName + '"';
            tmpBufferStream.write(next, next.length);
            tmpBufferStream.write("\r\n", 2);

            next = 'Content-Type: application/octet-stream';
            tmpBufferStream.write(next, next.length);
            tmpBufferStream.write("\r\n", 2);
            tmpBufferStream.write("\r\n", 2);

            tmpBufferStream.writeFrom(photoStream, photoFile.fileSize);
            tmpBufferStream.write("\r\n", 2);

            for(var p in aParams) {
                tmpBufferStream.write(boundary, boundary.length);
                tmpBufferStream.write("\r\n", 2);
                var next = 'Content-Disposition: form-data; name="' + p + '"';
                tmpBufferStream.write(next, next.length);
                tmpBufferStream.write("\r\n", 2);
                tmpBufferStream.write("\r\n", 2);
                var param = aParams[p] + "";
                var inputStream = converter.convertToInputStream(param);
                tmpBufferStream.writeFrom(inputStream, inputStream.available());
                //tmpBufferStream.write(param, param.length);
                tmpBufferStream.write("\r\n", 2);
            }

            tmpBufferStream.write(boundary, boundary.length);
            tmpBufferStream.write("--", 2);
            tmpBufferStream.write("\r\n", 2);
            tmpBufferStream.write("\r\n", 2);
            tmpBufferStream.close();
            tmpFileStream.close();

            url = this.endpoint;
            this.req = Components.classes[XMLHTTPREQUEST_CONTRACTID].createInstance(nsIXMLHttpRequest);
            var req = this.req;
            req.open("POST",url,true);
            req.setRequestHeader("Content-Type", contentType);

            var inst = this;
            var onReadyStateFunc = function(aEvt) {
                if(req.readyState == 4) {
                    //alert("\n" + req.responseText + "\n");
                    dump("\n" + req.responseText + "\n");
                    try {
                        if(req.status/100 == 2) {
                            var parser = Components.classes[DOMPARSER_CONTRACTID].createInstance(nsIDOMParser);
                            var dom = parser.parseFromString(req.responseText, "text/xml");
                            aListener.onResult(dom);
                        }
                        else {
                            aListener.onError(req.status);
                        }
                    }
                    catch(e) {
                        dump(e);
                        var error = Components.classes[FLOCK_ERROR_CONTRACTID].createInstance(flockIError);
                        aListener.onError(error.XML_HTTP_REQUEST_ERROR);
                    }
                }
                dump(req.readyState);
            };
            req.onreadystatechange = onReadyStateFunc;

            var tmpInputStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                .createInstance(Components.interfaces.nsIFileInputStream);
            tmpInputStream.init(tmpFile, 1, 0644, 0);

            var tmpInputBufferStream = Components.classes["@mozilla.org/network/buffered-input-stream;1"]
                .createInstance(Components.interfaces.nsIBufferedInputStream);

            tmpInputBufferStream.init(tmpInputStream, 65536 * 4);
            req.send(tmpInputBufferStream);
        }
        catch(e) {
            dump(e);
            var error = Components.classes[FLOCK_ERROR_CONTRACTID].createInstance(flockIError);
            aListener.onError(error.INVALID_PHOTO_UPLOAD);
        }
    },
}

function PhotoUploader() {
}
