/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is SpellBound code.
 *
 * The Initial Developer of the Original Code is
 * Robert Strong <moz@exchangecode.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Frank DeLicce (Ausdilicce) - inline spellcheck used with his permission
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gAtomService;

/* min & max of width & height for a resizable dialog */
var gDialogDims = {
  minHeight: null,
  minWidth:  null,
  maxHeight: null,
  maxWidth:  null
};

/* Just about everything encapsulated except for the EdSpellCheck and inline code */
var sbSpellCheck = {
  allowCodeToRun: true,  // will short circuit the function code if we need to
  editor:         null,  // a pointer to the current editor
  editorElement:  null,  // a pointer to the current editor element
  editorHasFocus: false, // self explanatory
  enabled:        true,  // in-line spell checker state
  enNode:         '',    // a holder node for when changing in-line spell checker states and debug states
  isHTML:         false, // are we checking html
  selEnd:         null,  // selectionEnd when performing selection checking (allows non modal)
  selStart:       null,  // selectionStart when performing selection checking (allows non modal)
  spellChecker:   null,  // will be the spell checker interface
  spellManager:   '',    // the spell checker manager (gets the right dictionaries for us)
  timer:          null,  // makes sure the code runs on the last keypress only (for fast typists)

  load: function() {
    if (window.arguments[3]) {
      var elements;
      this.editorElement = document.getElementById("content-frame");
      switch (this.getSpellCheckType()) {
        case 0:   // unsupported - should never hit this at this stage
          return;
        case 100: // textarea - handled by case 110
        case 101: // input - handled by case 110
        case 102: // textbox - handled by case 110
        case 103: // html:textarea - handled by case 110
        case 104: // html:input - handled by case 110
        case 110: // generic case for elements with a value property
          // selection checking
          if (window.arguments[3][0].selectionStart + 2 < window.arguments[3][0].selectionEnd) {
            this.selStart = window.arguments[3][0].selectionStart;
            this.selEnd = window.arguments[3][0].selectionEnd;
            this.setupSpellBoundSpellCheck(window.arguments[3][0].value.substring(this.selStart,
                this.selEnd));
          } else {
            this.setupSpellBoundSpellCheck(window.arguments[3][0].value);
          }
          break;
        case 200:  // editor
          this.editorElement = window.arguments[3][0].ownerDocument.defaultView.frameElement;
          this.editor = this.editorElement.getEditor(this.editorElement.contentWindow);
          this.setupRegularSpellCheck();
          break;
        case 201: // midas
          this.isHTML = true;
          var tmpNode = window.arguments[3][0];
          while (tmpNode.nodeName.toLowerCase() != "html")
              tmpNode = tmpNode.parentNode;
          elements = tmpNode.getElementsByTagName("head");
          var sTmp = elements[0].innerHTML;
          elements = this.editorElement.contentWindow.document.getElementsByTagName("head");
          elements[0].innerHTML = sTmp;
          elements = tmpNode.getElementsByTagName("body");
          this.setupSpellBoundSpellCheck(elements[0].innerHTML);
          break;
        case 202:  // editor passed as an argument
          this.editorElement = window.arguments[3][0];
          this.editor = this.editorElement.getEditor(this.editorElement.contentWindow);
          this.setupRegularSpellCheck();
          break;
        case 300: // extensions passing a string
          this.setupSpellBoundSpellCheck(window.arguments[3][0]);
          break;
        case 301: // extensions passing an assoc array with text/html
                  // remainder is handled by case 302
          this.isHTML = true;
        case 302: // extensions passing an assoc array with text/plain
          if (typeof window.arguments[3]["head"] != "undefined") {
            elements = this.editorElement.contentWindow.document.getElementsByTagName("head");
            elements[0].innerHTML = window.arguments[3]["head"];
          }
          this.setupSpellBoundSpellCheck(window.arguments[3]["body"]);
          break;
      }
    } else {
      // window.arguments[3] is empty - attempt a regular spellcheck
      this.setupRegularSpellCheck();
    }
    if (window.arguments[4] && window.arguments[4]=="work_please") { this.initSpellCheck(); }
    SetWindowLocation();
    return;
  },

  setupRegularSpellCheck: function() {
    var element = document.getElementById("content-frame");
    element.parentNode.removeChild(element);
    element = document.getElementById("appcontent");
    element.parentNode.removeChild(element);
    element = document.getElementById("vbox-spellui-flex");
    element.parentNode.removeChild(element);
    element = document.getElementById("spacer-appcontent");
    element.parentNode.removeChild(element);
    element = document.getElementById("resizer-spellcheck");
    element.parentNode.removeChild(element);
    window.sizeToContent();
  },

  setupSpellBoundSpellCheck: function(sBody) {
    var element = document.getElementById("appcontent");
    element.height = getSBPrefs().getIntPref("editor.height", 160) + "px";
    element = document.getElementById("groupbox-noattrs");
    element.setAttribute("id", "groupbox-attrs");

    this.editorElement.setAttribute("class", "show-editor");
    try {
      this.editorElement.makeEditable('htmlmail',false);
    } catch (e) {
      alert(e);
      return;
    }
    this.editor = this.editorElement.getEditor(this.editorElement.contentWindow);
    this.editor.QueryInterface(Components.interfaces.nsIEditorMailSupport);
    this.editor.QueryInterface(Components.interfaces.nsIPlaintextEditor);
    this.editor.QueryInterface(Components.interfaces.nsIHTMLEditor);
    this.editor.enableUndo(false);

    if (this.isHTML) {
      this.editorElement.contentWindow.document.body.innerHTML = sBody;
    } else {
      this.styleMisSpelled();
      this.editor.insertTextWithQuotations(sBody);
      if (getSBPrefs().getBoolPref("editor.misspelled.enabled", true)) {
        this.initSpellCheck();
        gSpellChecker.UninitSpellChecker();
        try {
          element = document.getElementById("AddToDictionary");
          element.setAttribute("oncommand", "AddToDictionary(); removeMisSpell();");
          element = document.getElementById("RecheckPage");
          element.setAttribute("oncommand", "removeMisSpell(); addMisSpell(); Recheck();");
          element = document.getElementById("EditDictionary");
          element.setAttribute("oncommand", "EditDictionary(); onEditDictionary();");
        } catch (e) { alert(e); }
      }
    }

    window.sizeToContent();
    gDialogDims.minWidth = window.outerWidth;
    gDialogDims.minHeight = window.outerHeight;
    gDialogDims.maxWidth = -1;
    gDialogDims.maxHeight = -1;
    setInterval('sbSpellCheck.fixWindowSize(gDialogDims.minWidth, gDialogDims.minHeight, gDialogDims.maxWidth, gDialogDims.maxHeight)', 500);
  },

  styleMisSpelled: function() {
    var rules = "body {\n" +
            "  font-family: -moz-fixed;\n" +
            "  margin: 0 2px 0 2px;\n"
  
    if (!getSBPrefs().getBoolPref("editor.display.wrap", true))
      rules += "  white-space: nowrap;\n"

    if (!this.isHTML) {
      rules += "  color: " + getSBPrefs().getCharPref("editor.display.foreground_color", "#000000") + ";\n"
      rules += "  background-color: " + getSBPrefs().getCharPref("editor.display.background_color", "#FFFFFF") + ";\n"
    }

    rules += "}\n";

    if (getSBPrefs().getBoolPref("editor.misspelled.enabled", true) && !this.isHTML) {

      rules += "span[_moz_misspelled='true'] {\n" +
          "  white-space: nowrap;\n";

      if (getSBPrefs().getBoolPref("editor.misspelled.display.underline", true))
        rules += "  background: url(chrome://spellbound/skin/underlines/" +
                 getSBPrefs().getCharPref("editor.misspelled.display.foreground_color", "#FF0000").substring(1).toLowerCase() + 
                 ".gif) repeat-x 100% 80%;\n" +
                 "  padding-bottom: 4px;\n";

      rules += "  background-color: " + getSBPrefs().getCharPref("editor.misspelled.display.background_color", "#FFFFFF") + ";\n";

      rules += "  color: " + getSBPrefs().getCharPref("editor.misspelled.display.foreground_color", "#FF0000") + ";\n";

      if (getSBPrefs().getBoolPref("editor.misspelled.display.bold", false))
        rules += "  font-weight: bold;\n";

      if (getSBPrefs().getBoolPref("editor.misspelled.display.italic", false))
        rules += "  font-style: italic;\n";

      rules += "}\n";
    }
    var head = this.editor.document.getElementsByTagName("HEAD");
    var style = this.editorElement.contentWindow.document.createElement("style");
    style.setAttribute("type", "text/css");
    style.innerHTML = rules;
    // use insertBefore to allow extensions to override the style
    head[0].insertBefore(style, head[0].childNodes[0]);
  },

  fixWindowSize: function(minWidth, minHeight, maxWidth, maxHeight) {
    if (typeof(window.windowState) != 'undefined' && window.windowState == STATE_NORMAL) {
      var nWidth = (minWidth > -1 && window.outerWidth < minWidth) ? minWidth : window.outerWidth;
      var nHeight = (minHeight > -1 && window.outerHeight < minHeight) ? minHeight : window.outerHeight;
      nWidth = (maxWidth > -1 && window.outerWidth > maxWidth) ? maxWidth : nWidth;
      nHeight = (maxHeight > -1 && window.outerHeight > maxHeight) ? maxHeight : nHeight;
      if (nWidth != window.outerWidth || nHeight != window.outerHeight) {
        window.resizeTo(nWidth, nHeight);
      }
    }
  }, 

  fillEditorContextMenu: function(event, contextMenuNode) {
    if (event.target != contextMenuNode)
      return;

    // Loop through all children to hide disabled items
    var children = contextMenuNode.childNodes;
    if (children) {
      var count = children.length;
      for (var i = 0; i < count; i++)
        this.HideDisabledItem(children[i]);
    }

    // Remove separators if all items in immediate group above are hidden
    // A bit complicated to account if multiple groups are completely hidden!
    var haveUndo =
      this.IsMenuItemShowing("cMenu_undo") ||
      this.IsMenuItemShowing("cMenu_redo");

    var haveEdit =
      this.IsMenuItemShowing("cMenu_cut")   ||
      this.IsMenuItemShowing("cMenu_copy")  ||
      this.IsMenuItemShowing("cMenu_paste") ||
      this.IsMenuItemShowing("cMenu_pasteNoFormatting") ||
      this.IsMenuItemShowing("cMenu_delete");

//    var haveSpelling =
//      this.IsMenuItemShowing("cMenu_spell");

    this.ShowMenuItem("undoredo-separator", haveUndo && haveEdit);

    this.ShowMenuItem("edit-separator", haveEdit || haveUndo);
// future
//    this.ShowMenuItem("spell-separator", haveSpelling);
    // Note: Item "cMenu_selectAll" is ALWAYS enabled so there is no seperator named after
    // selectAll. Instead it is named after the next item which may or may not be enabled
  },

// TODO: clean this up along with the following function as well
  disableSBControls: function() {
    if (this.editorHasFocus)
      return;

    gMisspelledWord = "";
    this.editorHasFocus = true;
    try {
      var element = document.getElementById("CheckWord");
      element.disabled = true;

      element = document.getElementById("SuggestedList");
      element.removeAttribute("onselect");
      element.removeAttribute("ondblclick");
      ClearListbox(element);
      element.disabled = true;

      element = document.getElementById("MisspelledWord");
      element.value = "";
      element.disabled = true;

      element = document.getElementById("ReplaceWordInput");
      element.value = "";
      element.disabled = true;

      element = document.getElementById("Replace");
      if (element.getAttribute("default") == "true") element.removeAttribute("default");
      element.disabled = true;

      element = document.getElementById("Ignore");
      if (element.getAttribute("default") == "true") element.removeAttribute("default");
      element.disabled = true;

      element = document.getElementById("ReplaceAll");
      if (element.getAttribute("default") == "true") element.removeAttribute("default");
      element.disabled = true;

      element = document.getElementById("IgnoreAll");
      if (element.getAttribute("default") == "true") element.removeAttribute("default");
      element.disabled = true;

      element = document.getElementById("UserDictionary");
      element.disabled = true;

      element = document.getElementById("AddToDictionary");
      if (element.getAttribute("default") == "true") element.removeAttribute("default");
      element.disabled = true;

      element = document.getElementById("EditDictionary");
      element.disabled = true;

      element = document.getElementById("RecheckPage");
      element.setAttribute("oncommand", "sbSpellCheck.enableSBControls(); removeMisSpell(); addMisSpell(); Recheck();");
      gMisspelledWord = "";

      element = document.getElementById("Close");
      if (element.getAttribute("default") == "true") element.removeAttribute("default");
    } catch (e) { }
  },

  // TODO: clean this up along with the previous function as well
  enableSBControls: function() {
    this.editorHasFocus = false;
    try {
      var element = document.getElementById("CheckWord");
      element.disabled = false;

      element = document.getElementById("EditDictionary");
      element.disabled = false;

      element = document.getElementById("SuggestedList");
      element.setAttribute("onselect", "SelectSuggestedWord();");
      element.setAttribute("ondblclick", "Replace();");
      element.disabled = false;

      element = document.getElementById("MisspelledWord");
      element.disabled = false;

      element = document.getElementById("ReplaceWordInput");
      element.disabled = false;

      element = document.getElementById("Replace");
      element.disabled = false;

      element = document.getElementById("Ignore");
      element.disabled = false;

      element = document.getElementById("ReplaceAll");
      element.disabled = false;

      element = document.getElementById("IgnoreAll");
      element.disabled = false;

      element = document.getElementById("UserDictionary");
      element.disabled = false;

      element = document.getElementById("AddToDictionary");
      element.disabled = false;

      element = document.getElementById("RecheckPage");
      element.setAttribute("oncommand", "removeMisSpell(); addMisSpell(); Recheck();");
    } catch (e) { }
  },

  initSpellCheck: function() {
    // called for EVERY spell check without an editor
    this.editor.addEditActionListener(SBeditActionListener);

    try {
      gSpellChecker = Components.classes['@mozilla.org/editor/editorspellchecker;1'].createInstance(Components.interfaces.nsIEditorSpellCheck);
      this.spellManager = Components.classes["@mozilla.org/spellchecker/i18nmanager;1"].createInstance(Components.interfaces.mozISpellI18NManager);
      gSpellChecker.setFilter(Components.classes["@mozilla.org/editor/txtsrvfiltermail;1"].createInstance(Components.interfaces.nsITextServicesFilter));
      gSpellChecker.InitSpellChecker(this.editor, true);

      var curLang = '';
      try {
        curLang = gSpellChecker.GetCurrentDictionary();
      } catch (e) { }
      var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

      try {
        curLang = pref.getCharPref("spellchecker.dictionary");
      } catch (e) { }
      if (curLang == '') {
        curLang = this.initLanguageArray();
      }

      if (curLang != "") {
        pref.setCharPref("spellchecker.dictionary", curLang);
        gSpellChecker.SetCurrentDictionary(curLang);
        addMisSpell();
      }
    } catch (e) { alert(e); }
  },

  initLanguageArray: function() {
    var curLang;
    var o1 = {};
    var o2 = {};
    try {
      gSpellChecker.GetDictionaryList(o1, o2);
    } catch (e) {
      dump("Failed to get DictionaryList!");
      alert("Failed to get DictionaryList!");
      return null;
    }
    var dictList = o1.value;
    var count  = o2.value;
    var languageBundle = document.getElementById("languageBundle");
    var regionBundle;
    if (languageBundle) regionBundle = document.getElementById("regionBundle");
    var menuStr2;
    var isoStrArray;
    var defaultItem = null;
    var langId;
    var i;

    for (i = 0; i < dictList.length; i++) {
      try {
        langId = dictList[i];
        isoStrArray = dictList[i].split("-");
        dictList[i] = new Array(2); // first subarray element - pretty name
        dictList[i][1] = langId;  // second subarray element - language ID
        if (languageBundle && isoStrArray[0]) dictList[i][0] = languageBundle.getString(isoStrArray[0].toLowerCase());
        if (regionBundle && dictList[i][0] && isoStrArray.length > 1 && isoStrArray[1]) {
          menuStr2 = regionBundle.getString(isoStrArray[1].toLowerCase());
          if (menuStr2) dictList[i][0] = dictList[i][0] + "/" + menuStr2;
        }
        if (!dictList[i][0]) dictList[i][0] = dictList[i][1];
      } catch (e) {
        dictList[i][0] = dictList[i][1];
      }
    }
    var results;
    if (dictList[0][1]) {
      results = dictList[0][1];
    } else {
      results = null;
    }
    return results;
  },

  doUpdate: function() {
    var sOutput;
    var sOrigInput;
    var results;

    SaveWindowLocation();

    if (window.arguments[3]) {
      switch (this.getSpellCheckType()) {
        case 0:   // unsupported - should never hit this at this stage
          break;
        case 100: // textarea handled by case 110
        case 101: // input handled by case 110
        case 102: // textbox handled by case 110
        case 103: // html:textarea handled by case 110
        case 104: // html:input - handled by case 110
        case 110: // generic handler
          sOutput = this.editor.outputToString("text/plain",2);
          // if there isn't a newline at the end of the in value remove
          // newlines from the end of the string the editor may have added
          if (/(\r\n|\r|\n)$/g.test(window.arguments[3][0].value) == false)
            sOutput = sOutput.replace(/(\r\n|\r|\n)$/g, "");
          // preserve spacing at end of original text that was spell checked
          // since the editor will remove it during modifications when
          // making spelling corrections
          if (this.selEnd && this.selEnd > 0) {
            sOrigInput = window.arguments[3][0].value.substring(this.selStart, this.selEnd)
            var spaces = "";
            var i;
            for(i = sOrigInput.length; i > 0; i--) {
              if (sOrigInput.substring(i-1, i) != " ")
                break;
              else
                spaces += " ";
            }
            for(i = sOutput.length; i > 0; i--) {
              if (sOutput.substring(i-1, i) != " ")
                break;
              else
                spaces = spaces.substring(0, spaces.length - 1);
            }
            sOutput = window.arguments[3][0].value.substring(0,
                this.selStart) + sOutput + spaces +
                window.arguments[3][0].value.substring(this.selEnd,
                window.arguments[3][0].value.length);
          }
          window.arguments[3][0].value = sOutput;
          return null;
        case 200: // editor
          // handled by the EdSpellCheck code
          return null;
        case 201: // midas
          var elements = window.arguments[3][0].ownerDocument.getElementsByTagName("body");
          elements[0].innerHTML = this.editor.rootElement.innerHTML;
          return null;
        case 300: // extensions passing a string
          sOrigInput = window.arguments[3][0];
          sOutput = this.editor.outputToString("text/plain",2);
          if (/(\r\n|\r|\n)$/g.test(sOrigInput) == false)
            sOutput = sOutput.replace(/(\r\n|\r|\n)$/g, "");
          results = window.arguments[4];
          results[0] = sOutput;
          return results;
        case 301: // extensions passing an assoc array with text/html
          results = window.arguments[4];
          results[0] = this.editor.rootElement.innerHTML;
          return results;
        case 302: // extensions passing an assoc array with text/plain
          sOrigInput = window.arguments[3]["body"];
          sOutput = this.editor.outputToString("text/plain",2);
          if (/(\r\n|\r|\n)$/g.test(sOrigInput) == false)
            sOutput = sOutput.replace(/(\r\n|\r|\n)$/g, "");
          results = window.arguments[4];
          results[0] = sOutput;
          return results;
      }
    }
    return null;
  },

  getSpellCheckType: function() {
    if (window.arguments[3][0]) {
      if (window.arguments[3][0].nodeName) // nodes passed by spellboundOverlay and extensions
        return sbGetNodeType(window.arguments[3][0]);
      else if (typeof window.arguments[3][0] == "string") // extensions passing a string
        return 300;
    // extensions passing multiple values in an assoc array
    } else if (typeof window.arguments[3]["type"] != "undefined") {
      if (window.arguments[3]["type"] == "text/html"
      && typeof window.arguments[3]["body"] != "undefined")
        return 301;
      else if (window.arguments[3]["type"] == "text/plain"
      && typeof window.arguments[3]["body"] != "undefined")
        return 302;
    }
    return null; // unsupported - should never hit this at this stage
  },

  IsItemOrCommandEnabled: function(item) {
    var command = item.getAttribute("command");
    if (command) {
    // If possible, query the command controller directly
      var controller = document.commandDispatcher.getControllerForCommand(command);
      if (controller)
        return controller.isCommandEnabled(command);
    }

    // Fall back on the inefficient observed disabled attribute
    return item.getAttribute("disabled") != "true";
  },

  HideDisabledItem: function(item) {
    item.hidden = !this.IsItemOrCommandEnabled(item);
  },

  ShowMenuItem: function(id, showItem) {
    var item = document.getElementById(id);
    if (item && !showItem) {
      item.hidden = true;
    }
  },

  IsMenuItemShowing: function(menuID) {
    var item = document.getElementById(menuID);
    if (item)
      return !item.hidden;

    return false;
  }

};

function onEditDictionary() {
  if (gMisspelledWord && sbSpellCheck.editor.selection.toString().length > 0 && !gSpellChecker.CheckCurrentWord(gMisspelledWord)) {
    try {
      var spans = sbSpellCheck.editorElement.contentDocument.getElementsByTagName("SPAN");
    } catch (e) { alert(e); }

    for (var i = 0; i < spans.length; i++) {
      try {
        if (spans[i].getAttribute("_moz_misspelled") && gMisspelledWord == spans[i].innerHTML.toString())
          spans[i].removeAttribute("_moz_misspelled");
      } catch (e) { alert(e); }
    }
    NextWord();
  }
}

function removeMisSpell() {
  try {
    var spans = sbSpellCheck.editorElement.contentDocument.getElementsByTagName("SPAN");
  } catch (e) { alert(e); }
  for (var i = 0; i < spans.length; i++) {
    try {
      if (spans[i].getAttribute("_moz_misspelled"))
        spans[i].removeAttribute("_moz_misspelled");
    } catch (e) { }
  }
}

function addMisSpell() {
  try {
    var curLang = gSpellChecker.GetCurrentDictionary();
    gSpellChecker.UninitSpellChecker();
    gSpellChecker.InitSpellChecker(sbSpellCheck.editor, false);
    gSpellChecker.SetCurrentDictionary(curLang);
    gMisspelledWord = gSpellChecker.GetNextMisspelledWord();

    if (gMisspelledWord) {
      sbSpellCheck.editor.beginTransaction();
      while(gMisspelledWord){
        EditorSetTextProperty('span','_moz_misspelled',true);
        gMisspelledWord = gSpellChecker.GetNextMisspelledWord();
      }
      sbSpellCheck.editor.endTransaction();
    }
    sbSpellCheck.editor.beginningOfDocument();
  } catch (e) { alert(e); }

}

/* Functions from the editor component */
function GetAtomService() {
  gAtomService = Components.classes["@mozilla.org/atom-service;1"]
    .getService(Components.interfaces.nsIAtomService);
}

function EditorGetTextProperty(property, attribute, value, firstHas, anyHas, allHas) {
  try {
    if (!gAtomService) GetAtomService();
    var propAtom = gAtomService.getAtom(property);

    sbSpellCheck.editor.getInlineProperty(propAtom, attribute, value, firstHas, anyHas, allHas);
  } catch (e) { }
}

function EditorSetTextProperty(property, attribute, value) {
  try {
  if (!gAtomService) GetAtomService();
  var propAtom = gAtomService.getAtom(property);

  sbSpellCheck.editor.setInlineProperty(propAtom, attribute, value);
  if ("gContentWindow" in window)
    window.gContentWindow.focus();
  } catch (e) { }
}

function EditorRemoveTextProperty(property, attribute) {
  try {
    if (!gAtomService) GetAtomService();
    var propAtom = gAtomService.getAtom(property);

    sbSpellCheck.editor.removeInlineProperty(propAtom, attribute);
    if ("gContentWindow" in window)
      window.gContentWindow.focus();
  } catch (e) { }
}

/* Below is modified code from Frank DeLicce (Ausdilicce) inline spell check */
var SBeditActionListener ={
  // this listener listens for edits to the editor
  WillCreateNode: function(aTag, aParent, aPosition) {},
  DidCreateNode:  function(aTag, aNode, aParent, aPosition, aResult) {},
  WillInsertNode: function(aNode, aParent, aPosition) {},
  DidInsertNode:  function(aNode, aParent, aPosition, aResult) {
    if (aNode.nodeName == "BR" && sbSpellCheck.allowCodeToRun) {
      if (sbSpellCheck.timer) clearTimeout(sbSpellCheck.timer);
      sbSpellCheck.timer = setTimeout(SBautoSpellReal,1,aNode,"BR","DidInsertNode")
    }
  },
  WillDeleteNode: function(aChild) {},
  DidDeleteNode:  function(aChild, aResult) {},
  WillSplitNode:  function(aExistingRightNode, aOffset) {},
  DidSplitNode:   function(aExistingRightNode, aOffset, aNewLeftNode, aResult) {},
  WillJoinNodes:  function(aLeftNode, aRightNode, aParent) {},
  DidJoinNodes:   function(aLeftNode, aRightNode, aParent, aResult) {},
  WillInsertText: function(aTextNode, aOffset, aString) {},
  DidInsertText:  function(aTextNode, aOffset, aString, aResult) {
    try {
      aTextNode.parentNode.removeAttribute("_moz_misspelled");
    } catch (e) { alert(e); }
    if (aResult == 0) {
      if (sbSpellCheck.editorHasFocus) {
        if (sbSpellCheck.timer) clearTimeout(sbSpellCheck.timer);
        sbSpellCheck.timer = setTimeout(SBautoSpellReal,1,aTextNode, aOffset + 1,"DidInsertText")
      }
    }
  },
  WillDeleteText: function(aTextNode, aOffset, aLength) { },
  DidDeleteText:  function(aTextNode, aOffset, aLength, aResult) {
    if (aResult == 0) {
      if (sbSpellCheck.editorHasFocus) {
        if (sbSpellCheck.timer) clearTimeout(sbSpellCheck.timer);
        sbSpellCheck.timer = setTimeout(SBautoSpellReal,1,aTextNode, aOffset + 1,"DidDeleteText")
      }
    }
  },
  WillDeleteSelection: function(aSelection) {},
  DidDeleteSelection:  function(aSelection) {}
};

function SBgetTextChild(aNode,side) {
  // this function goes thru a node looking for the text node child of it
  if (side == null) side = 'left'
  try {
    if (aNode.nodeType == aNode.TEXT_NODE) {
      return aNode;
    }
    // text node was not found so we recurse through the node
    if (aNode.hasChildNodes()) {
      for (var i=0; i < aNode.childNodes.length ; i++) {
        if (SBgetTextChild(aNode.childNodes[i],side).nodeType == aNode.TEXT_NODE) {
          aNode = aNode.childNodes[i];
          if (side == 'left') break;
        }
      }
    }
    return aNode;
  } catch (e) { }
  return null;
}

function SBautoSpellReal(theCurrentNode, theCurrentOffset, calledFrom) {
  if (!sbSpellCheck.editorHasFocus) return;

  var cn = sbSpellCheck.editor.selection.anchorNode;
  if (!cn) return;
  var holder = cn.nodeValue;

  var txte1;
  var anAtom = Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom('spell');
  if (cn.nodeName == "BODY") {
    sbSpellCheck.editor.selection.collapse(SBgetTextChild(cn.childNodes[sbSpellCheck.editor.selection.anchorOffset]),0)
    cn = sbSpellCheck.editor.selection.anchorNode;
      //return;
    if (cn.nodeName == "BR") {
      txte1 =  sbSpellCheck.editor.document.createTextNode("");
      cn.parentNode.insertBefore(txte1,cn);
      sbSpellCheck.editor.selection.collapse(txte1,0);
      return;
    }
  }
  sbSpellCheck.editor.beginPlaceHolderTransaction(anAtom);
  try {
    // note that theCurrentNode and cn are NOT the same in some circumstances
    if (theCurrentNode.nodeName == "BR") {
      // just check that the BR is not in a misspelled node.. if it is, take it out of the misspelled node
      if (SBisMisSpellNode(cn)){
        txte1 = sbSpellCheck.editor.document.createTextNode("");
        theCurrentNode.parentNode.insertBefore(txte1,theCurrentNode.parentNode.childNodes[2]);
        sbSpellCheck.editor.selection.collapse(theCurrentNode.parentNode,1);
        sbSpellCheck.editor.selection.extend(theCurrentNode.parentNode,theCurrentNode.parentNode.childNodes.length);
        sbSpellCheck.allowCodeToRun = false;
        SBeditorRemoveTextProperty('span','_moz_misspelled');
        sbSpellCheck.editor.selection.collapseToStart();
        sbSpellCheck.editor.selection.collapse(txte1,0);
        sbSpellCheck.allowCodeToRun = true;
      }
      sbSpellCheck.editor.endPlaceHolderTransaction();
      return;
    }
  } catch (e) {
    sbSpellCheck.editor.endPlaceHolderTransaction();
    return;
  }
  try {
    // lets save the anchorOffset no matter what
    var selection = sbSpellCheck.editor.selection;
    var EoS = getOffset(selection.anchorNode,selection.anchorOffset,true)
    // lets check the previous node and next node to see if they are misspelled
    var RnM = false;
    var LnM = false;
    var hNode = selection.anchorNode
    if (SBisMisSpellNode(hNode)) hNode = hNode.parentNode;
    // this puts hNode on the same 'level' as the text nodes on either side
    var curAo = selection.anchorOffset;
    var curAnLen = selection.anchorNode.textContent.length;
    try {
      // the left node
      if (curAo == 0) {
        // collapse on the previous node
        selection.collapse(SBgetTextChild(hNode.previousSibling,'right'),0);
        if (SBisMisSpellNode(selection.anchorNode)) {
          LnM = true;
        }
      }
    } catch (e) { }
    try {
      // now the right node
      if (curAo == curAnLen) {
        // collapse on the next node
        selection.collapse(SBgetTextChild(hNode.nextSibling,'left'),0);
        if (SBisMisSpellNode(selection.anchorNode)) {
          RnM = true;
        }
      }
    } catch (e) { }
    var lRange = sbSpellCheck.editor.document.createRange();
    lRange.setStart(hNode,0);
    if (LnM) lRange.setStart(SBgetTextChild(hNode.previousSibling),0);
    lRange.setEnd(SBgetTextChild(hNode),curAnLen);
    if (RnM) lRange.setEnd(SBgetTextChild(hNode.nextSibling),SBgetTextChild(hNode.nextSibling).textContent.length);
    sbSpellCheck.editor.selection.removeAllRanges();
    sbSpellCheck.editor.selection.addRange(lRange);
    var firstHas = { value: false };
    var anyHas = { value: false };
    var allHas = { value: false };
    EditorGetTextProperty("span", "_moz_misspelled", true, firstHas, anyHas, allHas);
    if (anyHas.value) {
      EditorRemoveTextProperty('span','_moz_misspelled');
      gotoOffset('start',EoS,true);
      cn = sbSpellCheck.editor.selection.anchorNode;  // this is now the anchorNode AFTER the span was removed
    }
    var ntxt = SBchkEachWord(cn);
    gotoOffset('start',EoS,true);
  } catch (e) { }
  sbSpellCheck.editor.endPlaceHolderTransaction();
}

function gotoOffset(node,offset,collapseOnEnd) {
  // this function *should* put the caret on the node <offset> characters from the start of the editor
  if (node == 'start') node = sbSpellCheck.editor.rootElement;
  if (!node.firstChild) {
    try {
      if (node.nodeName =="BR") {
        // we need to fix this
        //gotoOffset(node.previousSibling,node.previousSibling.textContent.length,true);
        sbSpellCheck.editor.selection.collapse(node,offset);
      } else {
        sbSpellCheck.editor.selection.collapse(node,offset);
      }
    } catch (e) {
      // this is here for nodes like brs.. do we have to create a text node and drop there ?, gotta be a better way
      sbSpellCheck.editor.selection.collapseToEnd();
    }
    return;
  }
  for (var i=0; i < node.childNodes.length ; i++) {
    if (node.childNodes[i].textContent.length > offset) {
      break;
    }
    if (node.childNodes[i].textContent.length == offset && collapseOnEnd) {
      break;
    }
    offset = offset - node.childNodes[i].textContent.length;
  }
  gotoOffset(node.childNodes[i],offset,collapseOnEnd)
}

function getOffset(node,offset,atStart) {
  // this function will return a number indicating the number of characters from the start of the editor to the caret (if positive)
  if (atStart && node.nodeType != 3) {
    // if the incoming node is not a text node, the anchor offset will be the index of the node to its parent, not the text offset within the node
    node = node.childNodes[sbSpellCheck.editor.selection.anchorOffset]
  }

  while (node.previousSibling) {
    offset += node.previousSibling.textContent.length;
    node = node.previousSibling;
  }
  if (node.parentNode.nodeName != "BODY") {
    offset = getOffset(node.parentNode,offset, false);
  }
  return offset;
}


function SBchkEachWord(node) {
  var txtTemp = node.textContent;
  if (!sbSpellCheck.wordBreaker)
    sbSpellCheck.wordBreaker = sbSpellCheck.spellManager.getUtil(gSpellChecker.GetCurrentDictionary());

  var wobegin = new Object;
  var woend = new Object;
  var sWord = {begin: wobegin, end:woend, txt: '' };
  var i = 0;
  var badlist = [''];
  var bl = 0;
  while (i != -1) {
    sbSpellCheck.wordBreaker.findNextWord(txtTemp,txtTemp.length,i,sWord.begin, sWord.end);
    i = sWord.end.value;
    if (i == -1) break;
    sWord.txt = txtTemp.substring(sWord.begin.value, sWord.end.value)
    if (sWord.txt.length > 0) {
      if (gSpellChecker.CheckCurrentWord(sWord.txt)) {
        var isThere = false;
        for (var j=0; j < badlist.length ; j++) {
          if (badlist[j] == sWord.txt) {
            isThere = true;
          }
        }
        if (!isThere) badlist[bl++] =  sWord.txt;
      }
    } else {
      // word is blank, don't do a thing
    }
  }
  // lets now take away any misspelled nodes within this node
  if (badlist[0] != '')
    SBsearchWithinNode(node, badlist);
}

function SBsearchWithinNode(node, WordArr){
  var pos, skip;
  skip=0;
  if (node.nodeType==3){
    for (var i=0; i < WordArr.length ; i++) {
      var newRE = new RegExp("\\b(" + WordArr[i] + "(?!\\<|\\w))","g");
      pos=node.data.search(newRE);
      if (pos>=0){
        try {
          sbSpellCheck.editor.selection.collapse(node,pos);
          sbSpellCheck.editor.selection.extend(node,pos + WordArr[i].length);
          EditorSetTextProperty('span','_moz_misspelled',true);
          // believe it or not .. we need *something* at the beginning of the rootElement of the editor
          // or the Real Spell Checker causes a GPF if a _moz_misspelled span exists on the first line
          var txte1 =  sbSpellCheck.editor.document.createTextNode("");
          if (sbSpellCheck.editor.rootElement.childNodes[0].nodeType == 3 && sbSpellCheck.editor.rootElement.childNodes[0].textContent.length == 0) {
          } else { sbSpellCheck.editor.rootElement.insertBefore(txte1,sbSpellCheck.editor.rootElement.childNodes[0]); }
        } catch (e) { }
        skip = 1;
      }
    }
  } else if (node.nodeType==1 && node.firstChild){
    for (var child=0; child < node.childNodes.length; ++child)
      child=child+SBsearchWithinNode(node.childNodes[child], WordArr);
  }
  return skip;
}

function SBisMisSpellNode(aNode) {
  // this function checks if a node is a misspelled (underlined) node by looking for a span node with the _moz_misspelled attribute
  var isMisSpelled = false;
  try {
    var firstHas = { value: false };
    var anyHas = { value: false };
    var allHas = { value: false };
    EditorGetTextProperty("span", "_moz_misspelled", true, firstHas, anyHas, allHas);
    if (anyHas.value) {
      isMisSpelled = true;
    }
  } catch (e) { }
  return isMisSpelled;
}
